#include "clock.h"
// v2.24 for radio

//#define DEBUG

#ifdef DEBUG
#warning !
#endif

//--------------------------------------
static void port_init(void)
{
DDRLCD=0xFF;
DDRSERIAL|=PIN_SERIAL;
PIN_T_STAT_0_INIT;
PIN_T_STAT_1_INIT;
_1_INIT_DDR;
_2_INIT_DDR;
_3_INIT_DDR;
_4_INIT_DDR;

PORT_POWER_DETECT|=_power_;
}
//**************************************
static void timer0_init(void){
	TCCR0=0x01;
}
//**************************************
static void timer2_init(void)
{
 TCCR2 = 0x00; 		  //stop
 ASSR  = (1<<AS2);	  //set async mode
 TCCR2 = 0x01; 		  //start
}
//**************************************


//**************************************

static void init_devices(void)
{
 cli(); //disable all interrupts
 port_init();
 timer0_init();
 MCUCR = 0x00;
 GICR  = 0x00;
 ACSR=_BV(ACD);
 TIMSK =(1<<TOIE2)|(1<<TOIE0); 
 sei(); //re-enable interrupts
}

// ***********************************************************
//   
static inline unsigned char EEPROM_READ(unsigned int uiAddress)
{
	/*     */
	while(EECR & (1<<EEWE));
	/*    */
	EEAR = uiAddress;
	/*       EERE */
	EECR |= (1<<EERE);
	/*      */
	return EEDR;
}

static void EEPROM_write(unsigned int uiAddress, unsigned char ucData)
{
//cli();
	if ( EEPROM_READ(uiAddress)!=ucData){
		/*     */
		while(EECR & (1<<EEWE));
		cli();
		/*     */
		EEAR = uiAddress;
		EEDR = ucData;
		/*  . 1   EEMWE */
		EECR |= (1<<EEMWE);
		/*       EEWE */
		EECR |= (1<<EEWE);
		sei();
		}
//sei();
}

void save_struct(uint8_t *a, uint8_t size, uint8_t adr, uint8_t err)
{
uint16_t *crc=(uint16_t *)(a+size-2);
		*crc=0;
		for(uint8_t j=0, i=adr; i<adr+size; i++, j++, a++)
			{
			if (j<size-2) (*crc)+=*a;
			EEPROM_write( i, *a);
			}
		eeprom_eer&=~err;
}

void read_struct(uint8_t *a, uint8_t size, uint8_t adr, uint8_t err)
{
uint16_t crc=0;
	 	for(uint8_t j=0, i=adr; i<adr+ size; i++, j++, a++)
			{*a=EEPROM_READ(i);
			if (j<size-2) crc+=*a;
			}
		if ( crc!= *( (uint16_t *)(a-2) ) ) eeprom_eer|=err;
}
//*********************************************
void read_data_EEPROM(void)
{
	eeprom_eer=0;
		
	read_struct((uint8_t *)&timer, sizeof(timer), EEPROM_timer, _BV(0));
	read_struct((uint8_t *)&setting, sizeof(setting), EEPROM_setting, _BV(1));
	read_struct((uint8_t *)&termostat_0, sizeof(termostat_0), EEPROM_termostat_0, _BV(2));
	read_struct((uint8_t *)&termostat_1, sizeof(termostat_1), EEPROM_termostat_1, _BV(3));
		
	temper_termostat_0=termostat_0.temperature[0];
	temper_termostat_1=termostat_1.temperature[0];
	
	if ( eeprom_eer & err_setting )
			setting.night_light=setting.day_light=lcd_light=5;
}

//*************** 1 Wire DALLAS ************************************************
void pullup_off(void){
	DDR_1wire&=~pwire0;
	DDR_1wire&=~pwire1;
	PORT_1wire&=~pwire0;
	PORT_1wire&=~pwire1;
}
//***********************************************
unsigned char readbit(void){
unsigned char i;
	cli();
	DDR_1wire|=out_1wire;//line down

//	asm("nop");
	asm("nop");
	asm("nop");//1.25us
	DDR_1wire&=~out_1wire;//line up through 1.25us
	_delay_us(12);
	i=PIN_1wire;
	sei();
	_delay_us(47);
	return i;
}
//***********************************************
static void readbyte(uint8_t *pw) {  
unsigned char i, r, read_byte[2]={0,0}, pinwire;

pullup_off();
for(i=0, r=1; i<8; i++){
	pinwire=readbit();
	if ( pinwire & pwire0 ) read_byte[0]|=r;
	if ( pinwire & pwire1 ) read_byte[1]|=r;
	r<<=1;
	}
PORT_1wire|=power_suplly_1w;
DDR_1wire|=power_suplly_1w;
pw[0]=read_byte[0];	
pw[1]=read_byte[1];
}
//***********************************************
static void writebit0(void){
	cli();
	DDR_1wire|=out_1wire;
	_delay_us(60);
	DDR_1wire&=~out_1wire;
	sei();
}
//***********************************************
void writebit1(void){
	cli();
	DDR_1wire|=out_1wire;
	_delay_us(3);
	DDR_1wire&=~out_1wire;
	sei();
	_delay_us(57);
}
//***********************************************
void writebyte(unsigned char byte){
unsigned char i;
pullup_off();
	for(i=0; i<8; i++){
		if (byte&_BV(0)) 
			writebit1();
			else
			writebit0();
		byte>>=1;
		}
	PORT_1wire|=power_suplly_1w;
	DDR_1wire|=power_suplly_1w;
}
//****************************************

void detectPresence(void){
unsigned char i;
	
	pullup_off();
	out_1wire=pwire0|pwire1;
	
	DDR_1wire|=pwire0;
	DDR_1wire|=pwire1;
	_delay_ms(0.48);
	
	cli();
	DDR_1wire&=~pwire0;
	DDR_1wire&=~pwire1;
	_delay_us(60);
	i=PIN_1wire;
	sei();
	
	if ( i & pwire0 ) {out_1wire&=~pwire0; PIN_T_STAT_0_OFF;}//   -    
	if ( i & pwire1 ) {out_1wire&=~pwire1; PIN_T_STAT_1_OFF;}

	if ( out_1wire == 0 ){
		count_com_1w=0; 
		return;
		}
	
	_delay_ms(0.42);
	i=PIN_1wire;
	if (!( i & pwire0 )) {out_1wire&=~pwire0; PIN_T_STAT_0_OFF;}//   -    
	if (!( i & pwire1 )) {out_1wire&=~pwire1; PIN_T_STAT_1_OFF;}
	
	if ( out_1wire == 0 ){
		count_com_1w=0; 
		return;
		}
			
	power_suplly_1w &= out_1wire;
	PORT_1wire|=power_suplly_1w;
	DDR_1wire|=power_suplly_1w;	
}

//****************************************
static void scratchpad_save(void){

		detectPresence();//  
		writebyte(0xCC);//SKIP ROM [CCh]
		writebyte(0x4E);//WRITE SCRATCHPAD [4Eh]
		writebyte(5);//	TH
		writebyte(5);//	TL
		writebyte(0x7f);//	Statys 12bit
		
		detectPresence();//   EEPROM
		writebyte(0xCC);//SKIP ROM [CCh]
		writebyte(0x48);//COPY SCRATCHPAD [48h]
		PORT_1wire|=power_suplly_1w;
		DDR_1wire|=power_suplly_1w;
}
//****************************************

void format_t(unsigned char p){
int t;
	t=((unsigned int)(tempir_msb[p]<<8))|tempir_lsb[p];
	temperatura[p]=(t*10)>>4;
}	
//**************************************************
static void izmerenie_t(void){

	if ( !count_com_1w ){//
		count_com_1w++;
		power_suplly_1w=0;
		detectPresence();//  
		return;
		}

	if ( count_com_1w == 1 ){
		count_com_1w++;

		writebyte(0xCC);//SKIP ROM [CCh]
		writebyte(0xB4);//READ POWER SUPPLY [B4h]
		pullup_off();
		power_suplly_1w=readbit();//     
		power_suplly_1w=~power_suplly_1w;//    ,  
		power_suplly_1w &= out_1wire;//pwire0|pwire1;
		PORT_1wire|=power_suplly_1w;
		DDR_1wire|=power_suplly_1w;
		return;
		}

	if ( count_com_1w == 2 ){	// 
		count_com_1w++;
		detectPresence();//
		return;
		}

	if ( count_com_1w == 3 ){
		count_com_1w++;
		writebyte(0xCC);//SKIP ROM [CCh]
		writebyte(0x44);//CONVERT T [44h]
		time_wire1=125;//1sek
		return;
		}

	if ( count_com_1w == 4 ){
		if (time_wire1<-120) count_com_1w++;
		return;
		}

	if ( count_com_1w == 5 ){// e
		count_com_1w++;
		detectPresence();//
		return;
		}

	if ( count_com_1w == 6 ){
		count_com_1w++;
		writebyte(0xCC);//SKIP ROM [CCh]
		writebyte(0xBE);//READ SCRATCHPAD [BEh]
		return;
		}

	if ( count_com_1w == 7 ){
		count_com_1w=0;
		readbyte(tempir_lsb);// 
		readbyte(tempir_msb);
		format_t(0);
		format_t(1);
		#define reg_termostat	1
		if ( !(regim==reg_termostat+3)){
			//--------------------------------------------------------
			if ( !(eeprom_eer & err_tstat_0) )
			{
			if (termostat_0.ohlagdenie==0)
				{
				if (temperatura[ds0] <= temper_termostat_0-termostat_0.gisterezis)	PIN_T_STAT_0_ON;// (_)   (-)  
				else
				if (temperatura[ds0] >= temper_termostat_0)	PIN_T_STAT_0_OFF;
				}else{
				if (temperatura[ds0] <= temper_termostat_0-termostat_0.gisterezis)	PIN_T_STAT_0_OFF;//
				else
				if (temperatura[ds0] >= temper_termostat_0)	PIN_T_STAT_0_ON;
				}
			}
			//--------------------------------------------------------
			}
		if ( !(regim==reg_termostat+6)){
			//---------------------------------------------------------
			if ( !(eeprom_eer & err_tstat_1) )
			{
			if (termostat_1.ohlagdenie==0)
				{
				if (temperatura[ds1] <= temper_termostat_1-termostat_1.gisterezis)	PIN_T_STAT_1_ON;// (_)   (-)  
				else
				if (temperatura[ds1] >= temper_termostat_1)	PIN_T_STAT_1_OFF;
				}else
				{
				if (temperatura[ds1] <= temper_termostat_1-termostat_1.gisterezis)	PIN_T_STAT_1_OFF;// (_)   (-)  
				else
				if (temperatura[ds1] >= temper_termostat_1)	PIN_T_STAT_1_ON;
				}
			}
			//---------------------------------------------------------
			}
		}

}
//*************** END 1 Wire *********************
//***********************************************/
static void change_light_var(void)
{
		if ( time_flag & _BV(fl_plus) )
		   {
		   if (++change_light>=lcd_light) time_flag &= ~_BV(fl_plus);//    
		   }
		   else
		   {
		   
		   if (change_light>1) change_light--; else time_flag|=_BV(fl_plus);//  
		   }
}
//**************************************
static uint8_t opred_light_current_num(void)
{
  char l;
  unsigned int a=0x100;
 	
	l=lcd_light;
	if (mask)
	 	{
		a>>=znakomesto;
		if ( mask&a ) l=change_light;
		}
		else	
		{
		time_flag&=~_BV(fl_plus);
		change_light=lcd_light;
		}
	return l;
}

//******************74hc595***********************************
void data_shift(unsigned char data)
{
	for(uint8_t i=8; i ; i--){//   8 
	if (data & _BV(7) )  PORTSERIAL|=_BV(DATA);
					else   PORTSERIAL&=~_BV(DATA); 
	data<<=1;
	PORTSERIAL|=_BV(SHIFT);
	PORTSERIAL&=~_BV(SHIFT);
	} 
}

static void display(void)
{
	PORTSERIAL|=_BV(STORAGE);// 
#ifdef katod 
	data_shift(~(1<<(znakomesto)));
#else
	data_shift(1<<(znakomesto));
#endif
	
	PORTSERIAL&=~_BV(STORAGE);
	PORTSERIAL|=_BV(STORAGE);
	PORTSERIAL&=~_BV(STORAGE);
	
	if (znakomesto==7){
	PORTSERIAL|=_BV(SHIFT);
	PORTSERIAL&=~_BV(SHIFT);
	}
	if (znakomesto==8)   
	#ifdef katod
		data_shift(0x0);
	#else
		data_shift(0xFF);
	#endif
	PORTSERIAL&=~(_BV(DATA)|_BV(SHIFT)|_BV(STORAGE));  
}

//*************************************************************
void skankey (void){
	unsigned char key_lokal=17;
	
	key=key_lokal;
	
	PORTSERIAL|=_BV(STORAGE);

	DDRLCD =0xF0;
	PORTLCD=0x0F;
	_delay_us(10);
	
	if (!str0)	    	key_lokal=0;
	else if (!str4)	key_lokal=4;
	else if (!str8)	key_lokal=8;
	else if (!str12)	key_lokal=12;
	else key_lokal=17;
	
	if ( key_lokal!=17 )
	{
		if (++anti_drebezg[0]==35)
			{
			key=key_lokal;
			}
		if (anti_drebezg[0]==anti_drebezg[2]){	//anti_drebezg[0]  
			anti_drebezg[0]=0;					//anti_drebezg[1] -   
			anti_drebezg[1]++;					//anti_drebezg[2]    
			if (anti_drebezg[1]==3) anti_drebezg[2]=45;//55;
			}	
		time_in_regim=25;
		if (pin_power_) time_in_regim=5;
	}
	else
	{
		anti_drebezg[0]=0;
		anti_drebezg[1]=0;
		anti_drebezg[2]=200;
	}
	#ifdef katod
	PORTLCD=0x00;
	#else
	PORTLCD=0xFF;
	#endif
	DDRLCD =0xFF;
}

//**************************************
void inc_dec_var_bit(unsigned char *var,unsigned char bit_n){
	if (PLUS)	 (*var)|= _BV(bit_n);
	if (MINUS) 	 (*var)&=~_BV(bit_n);
}

//**************************************
void inc_dec_var(unsigned char *var, unsigned char limit){
	if (PLUS){
		if ((*var)<limit)	(*var)++; 
		else *var=0;
		}
	if (MINUS){
		if (*var) (*var)--;
		else *var=limit;
		}
}

void inc_dec_var_temper(int16_t *var){
	if (PLUS){
			if ((*var+=5)>1200)	//T_stat->temperature[p_index]+=5; 
			//else 
			*var=0;
			}
		if (MINUS){
			if ( (*var-=5)<-1 ) //T_stat->temperature[p_index]-=5;
			//else 
			*var=1200;
			}
}
//**************************************
void mask_next_regim(unsigned int m, unsigned char next_F,unsigned char SET)
{
 	if (!anti_drebezg[0])  mask=m; 
	if (F_)   regim=next_F;
	if (SET_) regim=SET;
	key=17;
}

void next_regim(void){
	if (PLUS) {
		if (regim<217) 
		regim++;
		}
	if (MINUS) {
		if (regim>210)
		regim--;
		}
}


//**************************************
void key_action(void)
{//mask_next_regim( maska regima, if (key F_), if (key SET_) )
#define REG_SETTING_SAVE_EEP 18

	if (!regim) {
		p_index=0;
		if (F_) 
			regim=10;
			else
		if (SET_)
			regim=100;
			else
		if (MINUS) 
			regim=31;
			else
		if (PLUS) 
			regim=reg_termostat;
		return;
		}
	
	mask=0;

	if (regim==10) { mask_next_regim(0xFFFF,0,210);  return;}//setup
	
	//------------------------------------------------------------------
	if (regim==100){//     /
		inc_dec_var(&p_index, 11);
		mask_next_regim(0,0,0);
		return;
		}

	//---------------------------------------
	//#define reg_termostat	1 //   . izmerenie_t();
	if (regim==reg_termostat) {
		uint8_t j;
		inc_dec_var(&p_index,1);
		if (p_index==0) j=reg_termostat+1; else j=reg_termostat+4; 
		mask_next_regim(0x100,0,j);
		return;
		}
	//----------------------------------------
	if (regim==reg_termostat+1) { 
		mask_next_regim(0,0,reg_termostat+2);
		return;
	 	}
	if (regim==reg_termostat+2) { 
	 	inc_dec_var_temper(&temper_termostat_0);
		mask_next_regim(0b111000000,reg_termostat+1,reg_termostat+3);
		return;
	 	}
	if (regim==reg_termostat+3) { 
		if (PLUS) PIN_T_STAT_0_ON;
		if (MINUS) PIN_T_STAT_0_OFF;
		mask_next_regim(0b1000,0,reg_termostat+1);
		return;
	 	}
	//---------------
	if (regim==reg_termostat+4) { 
		mask_next_regim(0,0,reg_termostat+5);
		return;
	 	}
	if (regim==reg_termostat+5) { 
	 	inc_dec_var_temper(&temper_termostat_1);
		mask_next_regim(0b111000000,reg_termostat+4,reg_termostat+6);
		return;
	 	}
	if (regim==reg_termostat+6) { 
		if (PLUS) 	PIN_T_STAT_1_ON;
		if (MINUS) 	PIN_T_STAT_1_OFF;
		mask_next_regim(0b1000,0,reg_termostat+4);
		return;
	 	}
	//---------------------------------------------
	//--------------------t-stat-------------------
	if ( regim==212 ) {
		T_stat=&termostat_0; 
		next_regim();
		mask_next_regim(0xFFFF,0,80);
		return;
		}
	
	if ( regim==213 ) {
		T_stat=&termostat_1;
		next_regim();
		mask_next_regim(0xFFFF,0,80);
		return;
		}
	
	if (regim==80) {
	 	inc_dec_var(&(T_stat->kolvo_zons),kolvo_zon);
		if ( !T_stat->kolvo_zons ) T_stat->kolvo_zons=1;
	 	mask_next_regim(0b100000,88,81);
		return;
		}
	if (regim==81) {
	 	inc_dec_var(&(T_stat->ohlagdenie),1);
		p_index=0;
	 	mask_next_regim(0b11000,88,82);
		return;
		}
	
	if (regim==82) {
		uint8_t i;
	 	inc_dec_var(&p_index, T_stat->kolvo_zons);
		if ( p_index == T_stat->kolvo_zons ) i=87; else i=83;
	 	mask_next_regim(0x100,88,i);
		return;
		}
	if (regim==83) {
	 	inc_dec_var(&T_stat->time[p_index],23);
	 	mask_next_regim(0xC0,82,84);
		return;
		}
	if (regim==84) {
		uint8_t j;
		if ( p_index < T_stat->kolvo_zons-1 ) j=p_index+1; else j=0;
	 	inc_dec_var(&T_stat->time[j],23);
	 	mask_next_regim(0x18,82,85);
		return;
		}
	if (regim==85) {
		inc_dec_var_temper(&T_stat->temperature[p_index]);
	 	mask_next_regim(0b11100000,82,82);
		if (regim==82) p_index++;
		return;
		}
	if (regim==87) {
	 	inc_dec_var(&T_stat->gisterezis,99);
	 	mask_next_regim(0b11000,82,82);
		return;
		}
	if (regim==88) {
		if ( T_stat==&termostat_0 ){
			save_struct((uint8_t *)&termostat_0, sizeof(termostat_0), EEPROM_termostat_0, err_tstat_0);
			}
		else
			{
			save_struct((uint8_t *)&termostat_1, sizeof(termostat_1), EEPROM_termostat_1, err_tstat_1);
			}
		regim=212;
		return;
		}
	//--------------------END t-stat---------------
	
	//-----------------PORT-------------------------
	if (regim==211) {//120
		p_index=0;
		next_regim();
		mask_next_regim(0xFFFF,0,71);//Port  C	
		return;
		}
	if (regim==71) {								   
		if (SET_) if(++p_index==4) p_index=0;
		if (PLUS){
			switch(p_index)
				{
				case(0):
					_1_ON;
					break;
				case(1):
					_2_ON;
					break;
				case(2):
					_3_ON;
					break;
				case(3):
					_4_ON;
					break;
				}
			}
		if (MINUS){
			switch(p_index)
				{
				case(0):
					_1_OFF;
					break;
				case(1):
					_2_OFF;
					break;
				case(2):
					_3_OFF;
					break;
				case(3):
					_4_OFF;
					break;
				}
			}
		mask_next_regim(_BV(6-p_index),211,71);
		return;
		}
	//--------------------END PORT-----------------

	//--------------------CLOCK--------------------
	if (regim==210) {
		next_regim();
		mask_next_regim(0xFFFF,0,21); 
		return; 
		}
 	if (regim==21) {//set min hour
	 	inc_dec_var(&hour[h_min],59);
	 	mask_next_regim(0b110000,29,22);
		return;
		}
	if (regim==22) {//set hour
	 	inc_dec_var(&hour[h_hour],23);
	 	mask_next_regim(0b110000000,29,23);
		return;
		}
	if (regim==23) {//set day hour
	 	inc_dec_var(&hour[h_day],7-1);
	 	mask_next_regim(0b10000,29,24);
		return;
		}
	if (regim==24) {//set num hour
	 	inc_dec_var(&hour[h_num],31-1);
	 	mask_next_regim(0b11000000,29,25);
		return;
		}
	if (regim==25) {// set month hour
	 	inc_dec_var(&hour[h_month],12-1);
	 	mask_next_regim(0b11000,29,26);
		return;
		}
	if (regim==26) {//set hour sek
	 	if (MINUS){
			if (hour[h_sek]>31){ 
				hour[h_sek]=60;
				isr_1sek_=1;
				}else
				hour[h_sek]=0;
			clock_=0;
			}
	 	mask_next_regim(0b11000,29,27);
		return;
		}
	if (regim==27) {//set corection znak
	 	inc_dec_var_bit(&setting.corr_flag, fl_corr_znak_pl);
	 	mask_next_regim(0b10000000,29,28);
		return;
		}
	if (regim==28) {//set corection
	 	inc_dec_var(&setting.correction,sizeof(corr_in_lcd)/2-1);
	 	mask_next_regim(0b111000,29,29);
		return;
		}
	if (regim==29) {//set corection
	 	p_index=210;
	 	regim=REG_SETTING_SAVE_EEP;
		return;
		}
	//-----------------end cklock-----------------------	
	

	//------------------------------------
	if (regim==31) {//set number calss
	 	inc_dec_var(&budilnik_nomer,kolvo_bud-1);
	 	mask_next_regim(0x80,38,32);
		return;
	 	}
	if (regim==32) {//on off timer.calls
	 	inc_dec_var_bit(&timer.calls[ budilnik_nomer ][C_ON], 7 );
	 	mask_next_regim(0x20,31,33);
		return;
	 	}
	if (regim==33) {//set hour calss
	 	inc_dec_var(&timer.calls[ budilnik_nomer ][c_hour],23);
	 	mask_next_regim(0b1100000,31,34);
		return;
	 	}
	if (regim==34) {//set min timer.calls
	 	inc_dec_var(&timer.calls[ budilnik_nomer ] [c_min],59);
		mask_next_regim(0b11000,31,35);
		return;
	 	}

	if (regim==35) {
		inc_dec_var(&timer.calls[budilnik_nomer][na_hour],23); //c_hour
		mask_next_regim(0b110000000,31,36);
		return;
	 	}
	if (regim==36) {// set min
	 	inc_dec_var(&timer.calls[budilnik_nomer][na_min],59);//c_min
		mask_next_regim(0b1100000,31,37);
		return;
	 	}
	if (regim==37) {// set sek
	 	inc_dec_var(&timer.calls[budilnik_nomer][na_sek],59);//c_sek
		mask_next_regim(0b11000,31,31);
		return;
	 	}
			
	if (regim==38) {//save budilniks in eeprom
	
		save_struct((uint8_t *)&timer, sizeof(timer), EEPROM_timer, err_timer);
		regim=0;
		return;
		}
	//---------------- end timer.calls -------------------	

	
	//-------------SEE----------------
	if (regim==214){
		time_view_buf=3;
		setting.regim_state_time[Nreg_st]=0;
		next_regim();
		mask_next_regim(0xFFFF,0,91);//_t_SEE_
		return;
		}
	if (regim==91) {//F1 r.1-000 
	 	inc_dec_var( &setting.regim_state_time[Nreg_st], Nreg_st/2-1);
		p_index=214;
	 	mask_next_regim(0b100000000,REG_SETTING_SAVE_EEP,92);//BIT(8)
		return;
		}	
	if (regim==92) {//F1 r.1-000 
	 	inc_dec_var(&setting.regim_state_time[setting.regim_state_time[Nreg_st]*2],7);
	 	mask_next_regim(0b1000000,91,93);
		return;
		}	
	if (regim==93) {//r.1-000 F1
	 	inc_dec_var(&setting.regim_state_time[setting.regim_state_time[Nreg_st]*2+1],99);
	 	mask_next_regim(0b011000,91,91);
		return;
		}
	//-------------END SEE----------------

	//-------------------light--------------------------
	if (regim==215) {//125
		next_regim();
		mask_next_regim(0xFFFF,0,11); 
		return; 
		}
	if (regim==11) {
	 	inc_dec_var(&setting.day_light,sizeof(light_swith)-1);
		if (!setting.day_light) setting.day_light=1;
	 	mask_next_regim(0b1000000,17,14);
		return;
		}
	if (regim==14) {
	 	inc_dec_var(&setting.night_light,sizeof(light_swith)-1);
		if (!setting.night_light) setting.night_light=1;
	 	mask_next_regim(0b1000000,17,15);//noch L.5
		return;
		}
	if (regim==15) {
		inc_dec_var(&setting.tim_c_light,23);
		mask_next_regim(0b110000000,17,16);
		return;
		}
	if (regim==16) {
		inc_dec_var(&setting.tim_do_light,23);
		mask_next_regim(0b11000,17,17);
		return;
		}
	if (regim==17) {//set corection
	 	p_index=215;
	 	regim=REG_SETTING_SAVE_EEP;
		return;
		}
	
	if (regim==REG_SETTING_SAVE_EEP) {
		save_struct((uint8_t *)&setting, sizeof(setting), EEPROM_setting, err_setting);
		regim=p_index;
		return;
		}
	//---------------end__light--------------------

	
	//-------------ds18b20------------------
	if (regim==216) {
		next_regim();
		mask_next_regim(0xFFFF,REG_SETTING_SAVE_EEP,111);
		return; 
		}
	if (regim==111) {
		inc_dec_var(&setting.vid_t,1);
		mask_next_regim(0b10000000,216,112); 
		return;
		}
	if (regim==112) {
		mask_next_regim(0,REG_SETTING_SAVE_EEP,REG_SETTING_SAVE_EEP); 
		if (regim==REG_SETTING_SAVE_EEP) {
			scratchpad_save();
			p_index=216;
			}
		return;
		}
	//-----------end=ds18b20----------------
	
	//----------------------------------
	if (regim==217) {
	 	if (SET_){
			viewstate=0;
			
			for(uint8_t i=EEPROM_timer; i<EEPROM_timer+sizeof(timer) ; i++)
				EEPROM_write( i, 0);
			for(uint8_t i=EEPROM_termostat_0; i<EEPROM_termostat_0+sizeof(termostat_0) ; i++)
				EEPROM_write( i, 0);
			for(uint8_t i=EEPROM_termostat_1; i<EEPROM_termostat_1+sizeof(termostat_1) ; i++)
				EEPROM_write( i, 0);

			uint8_t *p;
			p=(uint8_t *)&setting;
			for(uint8_t i=0; i < sizeof(setting) ; i++){
		  		*(p++)=0;
				}
			setting.day_light=7;
			setting.night_light=4;
			setting.tim_c_light=23;
			setting.tim_do_light=6;
			setting.crc=(7+4+23+6);
			p=(uint8_t *)&setting;
			for(uint8_t i=EEPROM_setting; i<EEPROM_setting+sizeof(setting) ; i++, p++)
				EEPROM_write( i, *p);
			
			read_data_EEPROM();
		   }
		next_regim();
	 	mask_next_regim(0xFFFF,0,0);
		return;
		}
}

//**************************************
void result_in_bcd(int result)
{
    unsigned char dig_out=0;
	bcd_num1=0;
	bcd_num2=0;
	bcd_num3=0;
	bcd_num4=0;
	while (result >= 10000) {
	 result -= 10000;
 	 dig_out ++;
	}
	bcd_num0=dig_out;
    dig_out = 0;
	while (result >= 1000) {
	 result -= 1000;
 	 dig_out ++;
	}
	bcd_num1=dig_out;
    dig_out = 0;
	while (result >= 100)  {
 	 result -= 100;
	 dig_out ++;
	 }
	bcd_num2=dig_out;
	dig_out = 0;
	while (result >= 10)   {
	 result -= 10;
	 dig_out ++;
	 }
	bcd_num3=dig_out;
	bcd_num4= result;
}
//**************************************
static void cls_lcd_buffer(void){
unsigned char i;
 for(i=0;i<6;i++){
	lcd_buffer[i]=0;
	}
}

void load_str_in_lcdbuffer(prog_uint8_t* str_lcd){
unsigned char b,e,length;//str_lcd[0] = 0000 1111-, 1111 0000 -  
	length=pgm_read_byte_near(&str_lcd[0]);
	for( b=0x0F&(length), e=1 ; b<(0x0F&(length>>4)) ; b++, e++){
		lcd_buffer[b]=pgm_read_byte_near(&str_lcd[e]);
		}
}
//**************************************
void num2_bcd_in_lcd(unsigned char znak_mesto, unsigned char param_out)
{
 	result_in_bcd(param_out);
	lcd_buffer[znak_mesto]=font[bcd_num3];
	lcd_buffer[znak_mesto+1]  =font[bcd_num4];
}

//**************************************
void blank_nul(unsigned char mesto){
 if (lcd_buffer[mesto]==font[n0]) 
	lcd_buffer[mesto]=0;
}

//**************************************
void clock_in_lcd_gl(unsigned char znakomesto)
{
 	num2_bcd_in_lcd(znakomesto,hour[h_hour]);
	blank_nul(znakomesto);
	#ifdef miganie
 	if (time_flag&_BV(fl_sek_1_switch))
	#endif
		lcd_buffer[znakomesto+2]=font[f_seg_g];
	num2_bcd_in_lcd(znakomesto+3,hour[h_min]);
}
//**************************************
void clock_in_lcd_gl1(unsigned char znakomesto)
{
 	num2_bcd_in_lcd(znakomesto,hour[h_hour]);
	blank_nul(znakomesto);
	#ifdef miganie
 	if (time_flag&_BV(fl_sek_1_switch))
	#endif
		lcd_buffer[1]^=tochka;
	num2_bcd_in_lcd(znakomesto+2,hour[h_min]);
	//lcd_buffer[3]^=tochka;
}
//**************************************
void clock_in_lcd0(void){
 	lcd_buffer[0]=font[f_n];
	lcd_buffer[0]^=tochka;
	num2_bcd_in_lcd(1,hour[h_num]+1);
	blank_nul(1);
	lcd_buffer[3]=font[f_d];
	lcd_buffer[3]^=tochka;
	result_in_bcd(hour[h_day]+1);
	lcd_buffer[4]=font[bcd_num4];
}
//**************************************
void day_str(void){
	lcd_buffer[4]=pgm_read_byte_near(&h_day_str[hour[h_day]*2]);
	lcd_buffer[5]=pgm_read_byte_near(&h_day_str[hour[h_day]*2+1]);
}
//**************************************
static void clock_day_str(void){
	clock_in_lcd_gl1(0);
	day_str();
}
//**************************************
void lcd_hh_mm(unsigned char *param){
	num2_bcd_in_lcd(2,*param);
	lcd_buffer[3]^=tochka;
	num2_bcd_in_lcd(4,*(param-1));
}
//**************************************
void hh_mm_ss(uint8_t *param){
	num2_bcd_in_lcd(0,*(param+2));
	blank_nul(0);
	lcd_buffer[1]^=tochka;
	lcd_hh_mm(param+1);
}
//**************************************
static void num_month_day(void){//iln
	num2_bcd_in_lcd(0, hour[h_num]+1);
	blank_nul(0);
	lcd_buffer[1]^=tochka;
	num2_bcd_in_lcd(2, hour[h_month]+1);
	blank_nul(2);
	lcd_buffer[3]^=tochka;
	day_str();
}
//**************************************
void tempir_in_bcd(unsigned char t, unsigned char mesto, unsigned char pin_wire, unsigned char point){
unsigned char znak=0, n1=0,n2=0;

	if ( !(pin_wire & out_1wire) ){
	 	lcd_buffer[mesto]	=font[f_seg_g];
		lcd_buffer[mesto+1]	=font[f_seg_g];
		lcd_buffer[mesto+2]	=font[f_seg_g];
		}
		else
		{
	int te;
	te=temperatura[t];

		if ( te<0 ){
			te*=-1;
			znak=1;
			}

		result_in_bcd(te);// 15.7 104.3 -55.2 5.2 -5.3
		
		if (setting.vid_t==0)
		{
			if ( bcd_num1 ) n1=font[bcd_num1];
			else 
			if (znak) 	n1=font[f_seg_g];
			n2=font[bcd_num2];
		} 
		else
		{
			if ( bcd_num1 ) {
			n1=font[bcd_num1];
			n2=font[bcd_num2];
			}
			else
			{
			if ( bcd_num2 ){ 
			n2=font[bcd_num2]; 
			if ( znak ) n1=font[f_seg_g];
			}
			else
			if ( znak ) n2=font[f_seg_g];
			}
		}
		
		lcd_buffer[mesto]=n1;
		lcd_buffer[mesto+1]=n2;
		lcd_buffer[mesto+2]=font[bcd_num3];
		lcd_buffer[mesto+3]=font[bcd_num4];
		
		if (point) lcd_buffer[mesto+2]^=tochka;
			else if (bcd_num4>5) lcd_buffer[mesto+2]^=tochka;
		
		}
}
//**************************************/
void grad_celsia(void){
	lcd_buffer[5]=font[f_gr];
}
//**************************************/
static void one_tempir_0_in_lsd(void){
	tempir_in_bcd(ds0,1,pwire0,true);//012345678 //inl
	grad_celsia();
	lcd_buffer[0]=font[1];
	lcd_buffer[0]^=tochka;
}
//**************************************/
static void one_tempir_1_in_lsd(void){
	tempir_in_bcd(ds1,1,pwire1,true);//0-00.0 *c //inl
	grad_celsia();
	lcd_buffer[0]=font[2];
	lcd_buffer[0]^=tochka;
}
//*************************************
static void one_tempir_0_in_lsd0(void){
	tempir_in_bcd(ds0,1,pwire0,false);//
	lcd_buffer[0]=font[1];
	lcd_buffer[0]^=tochka;
	lcd_buffer[4]=font[f_gr];
	lcd_buffer[5]=font[f_C];
}
//**************************************/
static void one_tempir_1_in_lsd1(void){
	tempir_in_bcd(ds1,1,pwire1,false);//
	lcd_buffer[0]=font[2];
	lcd_buffer[0]^=tochka;
	lcd_buffer[4]=font[f_gr];
	lcd_buffer[5]=font[f_C];
}
//*************************************

void power_in_lcd(unsigned char pin_wire, unsigned char mesto){
unsigned char a,b;
	if ( !(pin_wire & out_1wire) ){
		a=font[f_seg_g];
		b=font[f_seg_g];
		}
		else
		if (power_suplly_1w & pin_wire )
		{
		a=font[f_P];
		b=font[f_A];	
		}
		else{
		a=font[f_o];
		b=font[f_n];
		}
	lcd_buffer[mesto]=a;
	lcd_buffer[mesto+1]=b;
}

//**************************************
void view_in_indikator(unsigned char view){

	switch (view)
	{
    case (0):
		clock_in_lcd_gl(0);//clock_in_lcd2();//hour-min
		break;
	case (1):
		hh_mm_ss(&hour[h_sek]);//hour-min-sek
		break;
	case (2):
		clock_day_str();//hour-min day
		break;
	case (3):
		num_month_day();//num_month day
		break;
    case (4):
		one_tempir_0_in_lsd();// _ -000 *C 1
		break;
    case (5):
		one_tempir_1_in_lsd();// _ -000 *C 2
		break;
	case (6):
		one_tempir_0_in_lsd0();
		break;
	case (7):
		one_tempir_1_in_lsd1();
		break;		
	}

}
//--------------------------------------
void indikator_manager(void){
unsigned char i;

 i=setting.regim_state_time[Nreg_st];
 if ( ++i>=(Nreg_st/2) ) i=0; //       .

	while( setting.regim_state_time[i*2+1] == 0 ){
		if ( i == setting.regim_state_time[Nreg_st]) {
			time_view_buf=1;
			return;
			}
		if ( ++i>=(Nreg_st/2) ) i=0;
		}
	setting.regim_state_time[Nreg_st]=i;
	time_view_buf=setting.regim_state_time[i*2+1];
	viewstate=setting.regim_state_time[i*2];
}

//**************************************/
static void data_lcd(void)
{
 	cls_lcd_buffer();
	
	if (regim){
		viewstate=0;
		time_view_buf = 3;
		}
	
	if (regim==0){
		if (eeprom_eer){
			load_str_in_lcdbuffer((prog_uint8_t *)str_err);
			if (eeprom_eer & err_timer ) lcd_buffer[2]=font[f_b];
			if (eeprom_eer & err_setting ) lcd_buffer[3]=font[f_H];
			if (eeprom_eer & err_tstat_0 ) lcd_buffer[4]=font[f_t];
			if (eeprom_eer & err_tstat_1 ) lcd_buffer[5]=font[f_t];
			}
		else
		view_in_indikator(viewstate);
		return;
	   }
	
	if (regim==100){
		view_in_indikator(p_index);
		return;
		}

	
	
	if  (regim==10){
		load_str_in_lcdbuffer((prog_uint8_t *)str_setup);
		return;
		}

	//-------------------------------------------
	if  ( regim==reg_termostat ){
		load_str_in_lcdbuffer((prog_uint8_t *)str_t_stat);
		lcd_buffer[0]=font[p_index+1];
		return;
		}
	
	if  (regim==reg_termostat+1){
		tempir_in_bcd(ds0,0,pwire0,false);
		lcd_buffer[3]=font[f_gr];
		lcd_buffer[4]=font[f_C];
		return;
		}
	if  ((regim>=reg_termostat+2)&&(regim<=reg_termostat+3)){
		result_in_bcd( temper_termostat_0 );
		lcd_buffer[1]=font[bcd_num2];
		lcd_buffer[2]=font[bcd_num3];
		if (bcd_num4==5) lcd_buffer[2]^=tochka;
		if (bcd_num1) {   lcd_buffer[0]=font[bcd_num1]; }

		lcd_buffer[3]=font[f_gr];
		lcd_buffer[4]	=font[f_P];
		if ( PIN_T_STAT_0_STATE ) lcd_buffer[5]=font[n1];
		   					   else lcd_buffer[5]=font[n0];
		return;
		}
	if  (regim==reg_termostat+4){
		tempir_in_bcd(ds1,0,pwire1,false);
		lcd_buffer[3]=font[f_gr];
		lcd_buffer[4]=font[f_C];
		return;
		}	
	if  ((regim>=reg_termostat+5)&&(regim<=reg_termostat+6)){
		result_in_bcd( temper_termostat_1 );
		lcd_buffer[1]=font[bcd_num2];
		lcd_buffer[2]=font[bcd_num3];
		if (bcd_num4==5) lcd_buffer[2]^=tochka;
		if (bcd_num1) {   lcd_buffer[0]=font[bcd_num1]; }

		lcd_buffer[3]=font[f_gr];
		lcd_buffer[4]	=font[f_P];
		if ( PIN_T_STAT_1_STATE ) lcd_buffer[5]=font[n1];
		   					   else lcd_buffer[5]=font[n0];
		return;
		}
	//--------------------------------------------------------	
		

	//-----------------t-stat-----------------------
	if (regim==212){
		load_str_in_lcdbuffer((prog_uint8_t *)str_t_stat);
		lcd_buffer[1]=font[1];
		return;
	    }
	if (regim==213){
		load_str_in_lcdbuffer((prog_uint8_t *)str_t_stat);
		lcd_buffer[1]=font[2];
		return;
	    }
	if ((regim>=80)&&(regim<=81)){
		load_str_in_lcdbuffer((prog_uint8_t *)str_kolvo_zon);
		lcd_buffer[3]=font[T_stat->kolvo_zons];
		if (T_stat->ohlagdenie) load_str_in_lcdbuffer((prog_uint8_t *)str_ohlag);
			else load_str_in_lcdbuffer((prog_uint8_t *)str_nagrev);
		return;
		}
	
	if ((regim>=82)&&(regim<=84)) {
		lcd_buffer[0]=font[p_index+1]^tochka;
		if (p_index<T_stat->kolvo_zons){
			num2_bcd_in_lcd(1,T_stat->time[p_index]);
			lcd_buffer[3]=font[f_seg_g];
			uint8_t j;
			if (p_index<T_stat->kolvo_zons-1) j=p_index+1; else j=0;
			num2_bcd_in_lcd(4,T_stat->time[j]);
		}else{
			load_str_in_lcdbuffer((prog_uint8_t *)str_gisterezis);
			//num2_bcd_in_lcd(4,T_stat->gisterezis);
			//lcd_buffer[4]^=tochka;
		}
		return;
		}
	
	if (regim==85) {
		lcd_buffer[0]=font[f_t]^tochka;
		if (p_index<T_stat->kolvo_zons){
			result_in_bcd(T_stat->temperature[p_index]);
			lcd_buffer[2]=font[bcd_num2];
			lcd_buffer[3]=font[bcd_num3];
			if (bcd_num4==5) lcd_buffer[3]^=tochka;
			if (bcd_num1) { lcd_buffer[5]^=tochka; lcd_buffer[1]=font[bcd_num1]; }
		}
		return;
		}
	
	if ((regim>85)&&(regim<=87)) {
		load_str_in_lcdbuffer((prog_uint8_t *)str_gisterezis);
			num2_bcd_in_lcd(4,T_stat->gisterezis);
			lcd_buffer[4]^=tochka;
		return;
		}
	
	//-----------------t-stat----------------------------
	
	//-----------------------------------------------
	if (regim==211){
		load_str_in_lcdbuffer((prog_uint8_t *)str_port);
		return;
	    }
	if (regim==71){
		load_str_in_lcdbuffer((prog_uint8_t *)str_pin_c);

		if ( _1_STATE ) lcd_buffer[2]=font[1]; else lcd_buffer[2]=font[f_seg_d];
		if ( _2_STATE ) lcd_buffer[3]=font[1]; else lcd_buffer[3]=font[f_seg_d];
		if ( _3_STATE ) lcd_buffer[4]=font[1]; else lcd_buffer[4]=font[f_seg_d];
		if ( _4_STATE ) lcd_buffer[5]=font[1]; else lcd_buffer[5]=font[f_seg_d];
		return;
	   	}
		
	//-----------------light-----------------------------	
	if  (regim==215){
		load_str_in_lcdbuffer((prog_uint8_t *)str_light);
		return;
		}
	if  (regim==11){
		lcd_buffer[0]=font[f_d]^tochka;
		lcd_buffer[1]=font[f_seg_g];
		lcd_buffer[2]=font[setting.day_light];
		lcd_light=setting.day_light;
		return;
		}
	if  (regim==14){
		lcd_buffer[0]=font[f_n]^tochka;
		lcd_buffer[1]=font[f_seg_g];
		lcd_buffer[2]=font[setting.night_light];
		lcd_light=setting.night_light;
		return;
		}
	if  ( (regim>14)&&(regim<17) ){
		num2_bcd_in_lcd(0,setting.tim_c_light);
		lcd_buffer[2]=font[f_seg_g];
		lcd_buffer[3]=font[f_seg_g];
		num2_bcd_in_lcd(4,setting.tim_do_light);
		return;
		}
		// end light
	
	
	//----------------------------
	if (regim==210){
		load_str_in_lcdbuffer((prog_uint8_t *)str_clock);
		return;
	    }
	if ((regim>=21)&&(regim<=22)){
	   	clock_in_lcd_gl(0);
		//clock_in_lcd0();
		return;
	    }
	if ((regim>=23)&&(regim<25)){
	   	clock_in_lcd0();
		return;
	    }
	if ( regim==25 ){
		load_str_in_lcdbuffer((prog_uint8_t *)str_monht);
		num2_bcd_in_lcd(4,hour[h_month]+1);
		blank_nul(4);
		return;
	   	}
		
	if (regim==26){
		hh_mm_ss(&hour[h_sek]);//hour-min-sek  
		return;
		}
	if ((regim>26)&&(regim<29)){//C, o, r, r.
		load_str_in_lcdbuffer((prog_uint8_t *)str_corr);
		if ( setting.corr_flag&_BV(fl_corr_znak_pl) ) lcd_buffer[1]=font[f_P]^tochka; 
	   	  else 							   	 lcd_buffer[1]=font[f_seg_g];   
		lcd_buffer[2]=font[n0]^tochka; 
		result_in_bcd( pgm_read_word_near(&corr_in_lcd[setting.correction]) );
		lcd_buffer[3]	=font[bcd_num2];
		lcd_buffer[4]	=font[bcd_num3];
		lcd_buffer[5]	=font[bcd_num4];
		return;
		}	

	//------------------------
	if ((regim>30)&&(regim<=32)){
		lcd_buffer[0]=font[budilnik_nomer / kolvo_bud_na_pin+1]^tochka;
		lcd_buffer[1]=font[budilnik_nomer % kolvo_bud_na_pin+1];
		
		if ( timer.calls[ budilnik_nomer ] [C_ON] & _BV(7) )
		lcd_buffer[3]=font[f_o]; else lcd_buffer[3]=font[f_seg_d];
		return;
		}
			
	if ((regim>=33)&&(regim<35)){
		
		lcd_hh_mm(&timer.calls[ budilnik_nomer ] [c_hour]);
		
		return;
		}	
	
	if ((regim>=35)&&(regim<=37)){
		hh_mm_ss(&timer.calls[ budilnik_nomer ] [na_sek]);
		return;
		}
	   

	//-----------------SEE-----------------------
	if (regim==214){//t see	
	    load_str_in_lcdbuffer((prog_uint8_t *)str_yst_see);
		return;
	    }
	if ((regim>90)&&(regim<94)) {
		lcd_buffer[0]=font[setting.regim_state_time[Nreg_st]+1];
		lcd_buffer[1]=font[f_r]^tochka;
		lcd_buffer[2]=font[setting.regim_state_time[setting.regim_state_time[Nreg_st]*2] ];
		lcd_buffer[3]=font[f_seg_g];
		result_in_bcd(setting.regim_state_time[setting.regim_state_time[Nreg_st]*2+1]);
		lcd_buffer[4]=font[bcd_num3];
		lcd_buffer[5]=font[bcd_num4];
		return;
		}
	
	//-----------------ds18b20-----------------------
	if (regim==216){
		load_str_in_lcdbuffer((prog_uint8_t *)str_ds18b20);
		return;
		}
	if (regim==111){
		if (setting.vid_t==0) 
			load_str_in_lcdbuffer((prog_uint8_t *)str_vid1_t);
			else
			load_str_in_lcdbuffer((prog_uint8_t *)str_vid2_t);
		return;
		}
	if (regim==112){
		//load_str_in_lcdbuffer((prog_uint8_t *)str_pit);
		lcd_buffer[0]=font[n1]^tochka;
		power_in_lcd(pwire0, 1);
		lcd_buffer[3]=font[n2]^tochka;
		power_in_lcd(pwire1, 4);
		return;
		}
	//----------------------------------------
	if (regim==217){
		load_str_in_lcdbuffer((prog_uint8_t *)str_default);
		return;
	    }
}



//**************************************
static void calls_verify(void)
{
unsigned char i;
	for(i=0;i<kolvo_bud;i++)
	if ( _BV(7) & timer.calls[i][C_ON] )				//  
	  if (timer.calls[i][c_hour]==hour[h_hour])		//
	   if (timer.calls[i][c_min]==hour[h_min])			//
		{
		budilnik_nomer=i;
		time_bud_pin[i/kolvo_bud_na_pin]=(uint32_t)timer.calls[i][na_hour]*3600;
		time_bud_pin[i/kolvo_bud_na_pin]+=timer.calls[i][na_min]*60;
		time_bud_pin[i/kolvo_bud_na_pin]+=timer.calls[i][na_sek];
		if (time_bud_pin[i/kolvo_bud_na_pin]){
			fl_calls_pin|=_BV(i/kolvo_bud_na_pin);
			if (!pin_power_)
				{
				switch(i/kolvo_bud_na_pin)
				{
				case(0):
					_1_ON;
					break;
				case(1):
					_2_ON;
					break;
				case(2):
					_3_ON;
					break;
				case(3):
					_4_ON;
					break;
				}
				}
			}

		}
}
//**************************************
static void calls_pin_verify(void){

for(uint8_t i=0; i<kolvo_bud_na_pin; i++)
	{
	if ( fl_calls_pin & _BV(i) )
		{
		if (--time_bud_pin[i]==0)
			{
			fl_calls_pin&=~_BV(i);
				switch(i)
				{
				case(0):
					_1_OFF;
					break;
				case(1):
					_2_OFF;
					break;
				case(2):
					_3_OFF;
					break;
				case(3):
					_4_OFF;
					break;
				}
			}
		}
	}
}



static void set_temper_termostat_0(uint8_t j){
if ( hour[h_hour] == termostat_0.time[j]) temper_termostat_0=termostat_0.temperature[j];
}

static void set_temper_termostat_1(uint8_t j){
if ( hour[h_hour] == termostat_1.time[j]) temper_termostat_1=termostat_1.temperature[j];
}

//**************************************
static void clock(void){

if ( isr_1sek_ )// 
{
correct=0;
calls_pin_verify();//   	
	hour[h_sek]++;
	if (hour[h_sek]>=60)
	{
   	hour[h_sek] =0;
	hour[h_min]++;
	if (hour[h_min]>=60)
		{
		if ( setting.corr_flag & _BV(fl_corr_znak_pl) )// 
			correct=setting.correction;// 
			else
			correct= 0x00-setting.correction;
	
		hour[h_min] =0;
		hour[h_hour]++;
		
		if (hour[h_hour]>=24)
			{
			hour[h_hour]=0;
			hour[h_day]++;
			hour[h_num]++;
			if (hour[h_day]>=7)	hour[h_day]=0;
			if (hour[h_num]>=num_in_month[ hour[h_month] ] )
				{
				hour[h_num]=0;
				hour[h_month]++;
				if (hour[h_month]>=12) 	{hour[h_month]=0;}
				}
			}
		
		if ( hour[h_hour] == setting.tim_c_light)	lcd_light=setting.night_light;
		if ( hour[h_hour] == setting.tim_do_light)	lcd_light=setting.day_light;
		
		for(uint8_t i=0; i<termostat_0.kolvo_zons; i++){
			set_temper_termostat_0(i);
			}
		for(uint8_t i=0; i<termostat_1.kolvo_zons; i++){
			set_temper_termostat_1(i);
			}
		
		// / 
		//           2:00     1  
		//          3:00    1  .
		if ( hour[h_month]==3-1 ){//  
			if ( hour[h_num]>24-1 )//  25  
				if ( hour[h_day]==7-1 )// 
					if (hour[h_hour]==2) hour[h_hour]=3;
			}
		if ( hour[h_month]==10-1 ){//  
			if ( hour[h_num]>24-1 )//  25  
				if ( hour[h_day]==7-1 )// 
					if (hour[h_hour]==3) hour[h_hour]=2;
			}
		}
	
	calls_verify();
	}
isr_1sek_=0;
}
 		
}
//****************************************
static void time_count(void){
static unsigned char t, sek;

	if ( t != clock_ )
		{//  0,0078125 .
		t=clock_;
		if (t>=64) time_flag|=_BV(fl_sek_1_switch); else time_flag&=~_BV(fl_sek_1_switch);
		time_wire1--;
		
		
		if (sek!=hour[h_sek]){// 
			sek=hour[h_sek];

			if (--time_view_buf == 0 ) indikator_manager();
			if (--time_in_regim == 0 )
						if (regim>=10) regim=0;
			
			}

		if (regim==0) { 
			if (pin_power_) {skankey(); if (key!=17)  key_action();}
			}
			else 
			if (power_save==1) TCCR0=0x01;
			
		
		if (regim>0)
			{
			static uint8_t time_mig=0;
			if (++time_mig>=light_time_mig[lcd_light])
				{
				time_mig=0;
				change_light_var();
				}
			}
		}
}


//**************************************
int main(void)
{
//uint8_t power_save=0;
	cli();
	read_data_EEPROM();
	timer2_init();
	init_devices();

while(1)
 {//while
	clock();
		/* */   
	if ( (power_save==0)||(regim) )
	{
		if (lcd_time_switch>=30)/* 2msek*/ 
			{
			lcd_time_switch=0x00;
			power_save=0;
			
			PORTSERIAL|=_BV(STORAGE);//  
				#ifdef katod
				PORTLCD=0x00;
				data_shift(0xFF);
				#else
				PORTLCD=0xFF;
				data_shift(0);
				#endif

			znakomesto++;
  	   		if  (znakomesto<6) {
				light=light_swith[opred_light_current_num()];//   
				if (pin_power_) light=27;
				skankey(); //    :)
				key_action();//   
				display();//  
				}
				else	
				{
				lcd_time_switch=25;

				if (regim){
					uint8_t l=setting.day_light;
					if ( setting.tim_do_light>setting.tim_c_light )
						{
						if ( (hour[h_hour] < setting.tim_do_light )&&( hour[h_hour] >= setting.tim_c_light ) ) 
							l=setting.night_light; else l=setting.day_light;
						}
						else
						{
						if ( (hour[h_hour] >= setting.tim_do_light )&&( hour[h_hour] < setting.tim_c_light ) ) 
							l=setting.day_light; else l=setting.night_light;
						}
					lcd_light=l;
					}
				data_lcd();
				znakomesto=0xFF;
				}
			}
	if (lcd_time_switch<5) izmerenie_t();	/* */
	}
	

	
	if (regim==0) 
	{
		if (pin_power_) 
		{
		
		if (power_save==0){
			power_save=1;
			TCCR0=0x00;//stop timer
			PORTSERIAL|=_BV(STORAGE);//  
			#ifdef katod
			PORTLCD=0x00;
			data_shift(0xFF);
			#else
			PORTLCD=0xFF;
			data_shift(0);
			#endif
			PORTSERIAL&=~_BV(DATA); 
			
			_1_OFF;
			_2_OFF;
			_3_OFF;
			_4_OFF;
			
			PIN_T_STAT_0_OFF;
			PIN_T_STAT_1_OFF;
			}
		
		MCUCR = BIT(SM1)|BIT(SM0)|BIT(SE);//Power Save ( ) 
		#ifndef DEBUG
		asm("sleep");
		#endif
		MCUCR=0;

		}
		else{
		if (power_save==1){
			power_save=0;
			TCCR0=0x01;
			
			if ( fl_calls_pin & _BV(0) ) _1_ON;
			if ( fl_calls_pin & _BV(1) ) _2_ON;
			if ( fl_calls_pin & _BV(2) ) _3_ON;
			if ( fl_calls_pin & _BV(3) ) _4_ON;
			}
		MCUCR = BIT(SE);//idle 
		#ifndef DEBUG
		asm("sleep");
		#endif
		MCUCR=0;
		}
	}
	time_count();

 }//while

}

//**************************************
ISR(SIG_OVERFLOW0){//0,000064
 	lcd_time_switch++;
	if (lcd_time_switch==light)
		#ifdef katod 
		PORTLCD=lcd_buffer[znakomesto];
		#else
		PORTLCD=~lcd_buffer[znakomesto];
		#endif
}
//**************************************
ISR(SIG_OVERFLOW2)
{
 clock_++;
 if (clock_==128)//128
	{	   	 		  	   	  				   	
	clock_=correct;
	isr_1sek_=1;
  	}
}
